#include <plib.h>

#define USE_AND_OR  // To enable AND_OR mask setting for I2C. 
#define SLAVE_ADDR 0xd0 // specific to MPU6050
#define ACC_XH 0x3b
#define ACC_XL 0x3c
#define ACC_YH 0x3d
#define ACC_YL 0x3e
#define ACC_ZH 0x3f
#define ACC_ZL 0x40
#define GYR_XH 0x43
#define GYR_XL 0x44
#define GYR_YH 0x45
#define GYR_YL 0x46
#define GYR_ZH 0x47
#define GYR_ZL 0x48

int X_GYRO_OFF, Y_GYRO_OFF, Z_GYRO_OFF;

// Wait by executing nops
void i2c_wait(unsigned int cnt){
    while(--cnt) { asm("nop"); asm("nop"); }
}

// Write a number of chars from data specified by num to the specified address
void i2c_write(char address, char data) {
    StartI2C1();    // Send start bit
    IdleI2C1();     // Wait to complete
    
    // send mpu6050 slave address
    while(1){
        MasterWriteI2C1( SLAVE_ADDR | 0 );
        IdleI2C1();
        if( !I2C1STATbits.ACKSTAT ) break;
    }
    // send target register address
    while(1){
        MasterWriteI2C1( address );
        IdleI2C1();
        if( !I2C1STATbits.ACKSTAT ) break;
    }
    // send 'num' bytes
    while(1){
        MasterWriteI2C1( data );
        IdleI2C1(); 
        if( !I2C1STATbits.ACKSTAT ) break;
    }
    
    StopI2C1(); // Send stop bit
    IdleI2C1(); // Wait to complete
}

// Read a char from the register specified by address
char i2c_read(char address) {
    char i2c_header[2];
    i2c_header[0] = SLAVE_ADDR | 0 ;    //device address & WR
    i2c_header[1] = address;                //register address

    StartI2C1();    //Send the Start Bit
    IdleI2C1();     //Wait to complete

    int i;
    for(i = 0; i < 2; i++)
    {
        MasterWriteI2C1( i2c_header[i] );
        IdleI2C1();     //Wait to complete

        // ACKSTAT is 0 when slave acknowledge, 
        if( I2C1STATbits.ACKSTAT )
            break;
    }
    
    // send start sequence again
    RestartI2C1();
    i2c_wait(10);
    IdleI2C1(); //Wait to complete

    //transmit read command
    MasterWriteI2C1( SLAVE_ADDR | 1 ); 
    IdleI2C1();     //Wait to complete

    // read some bytes back
    char data = MasterReadI2C1();
    IdleI2C1(); //Wait to complete
    
    StopI2C1(); //Send the Stop condition
    IdleI2C1(); //Wait to complete
    
    return data;
}


// Read accelerometer x value 
int readIMU_accX(){
    int h = i2c_read(ACC_XH);
    int l = 0x00ff & i2c_read(ACC_XL);
    return (h<<8) | l;
}

// Read accelerometer y value 
int readIMU_accY(){
    int h = i2c_read(ACC_YH);
    int l = 0x00ff & i2c_read(ACC_YL);
    return (h<<8) | l;
}

// Read accelerometer z value 
int readIMU_accZ(){
    int h = i2c_read(ACC_ZH);
    int l = 0x00ff & i2c_read(ACC_ZL);
    return (h<<8) | l;
}

// Read gyroscope x value 
int readIMU_gyrX(){
    int h = i2c_read(GYR_XH);
    int l = 0x00ff & i2c_read(GYR_XL);
    return ((h<<8) | l) - X_GYRO_OFF;
}

// Read gyroscope y value 
int readIMU_gyrY(){
    int h = i2c_read(GYR_YH);
    int l = 0x00ff & i2c_read(GYR_YL);
    return ((h<<8) | l) - Y_GYRO_OFF;
}

// Read gyroscope z value 
int readIMU_gyrZ(){
    int h = i2c_read(GYR_ZH);
    int l = 0x00ff & i2c_read(GYR_ZL);
    return ((h<<8) | l) - Z_GYRO_OFF;
}

void readIMU_acc(int* acc){
    acc[0] = readIMU_accX();
    //acc[1] = readIMU_accY();
    acc[2] = readIMU_accZ();
}

void readIMU_gyr(int* gyr){
   // gyr[0] = readIMU_gyrX();
    gyr[1] = readIMU_gyrY();
   // gyr[2] = readIMU_gyrZ();
}

// Sample the gyroscope over an extended period and average
// in order to find and subtract the bias
void calibrateGyroscope()
{
    int numSamples = 100;
    int xGyroSum = 0;
    int yGyroSum = 0;
    int zGyroSum = 0;
    int values[3];

    int i;
    for (i = 0; i < numSamples; i++){
        xGyroSum += readIMU_gyrX();
        yGyroSum += readIMU_gyrY();
        zGyroSum += readIMU_gyrZ();
    }
    
    X_GYRO_OFF = xGyroSum/numSamples;
    Y_GYRO_OFF = yGyroSum/numSamples;
    Z_GYRO_OFF = zGyroSum/numSamples;
}

